from dexter.retriever.dense.ColBERT.colbert.modeling.tokenization.query_tokenization import *
from dexter.retriever.dense.ColBERT.colbert.modeling.tokenization.doc_tokenization import *
from dexter.retriever.dense.ColBERT.colbert.modeling.tokenization.utils import tensorize_triples
