class Split:
    TEST = "test"
    TRAIN = "train"
    DEV = "dev"
    PRED = "predict"

class Dataset:
    AMBIGQA = "ambignq"
    WIKIMULTIHOPQA = "wikimultihopqa"
    FINQA = "finqa"
    TATQA = "tatqa"
    MUSIQUEQA="musiqueqa"
    OTTQA = "ottqa"
    StrategyQA = "strategyqa"

class Separators:
    TABLE_ROW_SEP = '\n'
    TABLE_COL_SEP = '|'

class DataTypes:
    TABLE = "table"
    TEXT = "text"
